<?php 

/* 
 * schema.php
 * Displays the schema for the specified server_id
 * 
 * Variables that come in as GET vars:
 *  - server_id
 *  - view (optional: can be 'attr' or empty. If 'attr', show that attribute)
 *  - attr (optional)
 */
 
require 'config.php';
require_once 'functions.php';

$server_id = $_GET['server_id'];
$view = $_GET['view'];

check_server_id( $server_id ) or 
	davedap_error( "Bad server_id: " . htmlentities( $server_id ) );
have_auth_info( $server_id ) or 
	davedap_error( "Not enough information to login to server. Please check your configuration." );
davedap_ldap_connect( $server_id ) or 
	davedap_error( "Coult not connect to LDAP server." );

$schema = get_schema( $server_id );

include 'header.php';

?>

<body>

<?php  if( ! $schema ) {

	davedap_error( "Could not retrieve schema from <b>" . htmlentities($servers[$server_id]['name']) . "</b>.<br />
		<br />
		This could happen for several reasons, the most probable of which are:
		<ul>
			<li>The server does not fully support the LDAP protocol.</li>
			<li>Your version of PHP does not correctly perform the query.</li>
		</ul>" );

} else { ?>

	<h3 class="title">Schema for server <b><?php echo htmlentities($servers[$server_id]['name']); ?></b></h3>

	<?php  flush(); ?>

	<?php
		
	if( $view == 'attr' )
	{
		$attr = $_GET['attr'];
		if( ! isset( $_GET['attr'] ) )
			davedap_error( "No such attr: " . htmlentities( $attr ) );

		?>

		<h3 class="subtitle">Attribute definition for <b><?php echo htmlentities( $attr ); ?></b></h3>

		<center>
		<h3>Attribute definition for '<?php echo htmlentities( $attr ); ?>'</h3>
		<table class="schema_attr">

		<?php
			
		if( is_array( $schema['attrs'][strtolower($attr)] ) )
		{
			$counter = 0;
			foreach( $schema['attrs'][strtolower($attr)] as $key => $val )
			{
				if( $key != 'val' && $val != null )  {
					$counter++;
					echo "<tr class=\"" . ($counter%2==0 ? 'even' : 'odd') . "\"><td>$key</td><td>$val</td></tr>\n";
				}
			}
		}
		else 
			davedap_error( "Bad schema entry for attribute: " . htmlentities( $attr ) );

		?>

		</table>
		</center>
		
		<?php
		
	} else { ?>
		
		<h3 class="subtitle"><?php echo count($schema['oclasses']); ?> objectClasses</h3>
		<small>Jump to an objectClass:</small>
		<select name="oclass_jumper"
			onChange="window.location.href='schema.php?server_id=<?php echo $server_id; ?>#'+this.value">
				
			<?php foreach( $schema['oclasses'] as $oclass => $desc ) { ?>
				<option value="<?php echo $oclass; ?>"><?php echo $desc['name']; ?></option>
			<?php } ?>
			
		</select>
		<br />

		<?php foreach( $schema['oclasses'] as $oclass => $attrs ) { ?>
			
			<small>[<a name="<?php echo $oclass; ?>" href="#" title="Head on up to the top.">top</a>]</small>
			<h4 class="oclass">objectClass <b><?php echo $attrs['name']; ?></b></h4>
			<h4 class="oclass_sub">OID <b><?php echo $attrs['oid']; ?></b></h4>
			<?php if( $attrs['description'] ) { ?>
				<h4 class="oclass_sub">Description <b><?php echo $attrs['description']; ?></b></h4>
			<?php } ?>
			<h4 class="oclass_sub">Inherits <b><?php echo $attrs['sup']; ?></b></h4>
			<br />
	
			<b>Required Attrbibutes</b>
			<ul class="schema">
			
			<?php 
				
			if( count( $attrs['must_attrs'] ) > 0 )
				foreach( $attrs['must_attrs'] as $attr ) 
					echo "<li><a href=\"schema.php?server_id=$server_id&amp;view=attr&amp;attr=" .
						rawurlencode( $attr ) . "\">" . htmlentities($attr) . "</a></li>\n";
			else				
				echo "(none)\n";

			?>

			</ul>
			<b>Optional Attrbibutes</b>
			<ul class="schema">

			<?php 
				
			if( count( $attrs['may_attrs'] ) > 0 )
				foreach( $attrs['may_attrs'] as $attr ) 
					echo "<li><a href=\"schema.php?server_id=$server_id&amp;view=attr&amp;attr=" .
						rawurlencode( $attr ) . "\">" . htmlentities($attr) . "</a></li>\n";
			else				
				echo "(none)\n";

			?>
				
			</ul>
			
		<?php  } /* End foreach objectClass */ ?>
	<?php } /* End else (displaying objectClasses */ ?>
<?php } /* End else (schema _is_ available) */ ?>

</body>

</html>
